from distutils.core import setup

setup(
    name='lazy_format',
    version='0.0.2',
    packages=['lazy_format'],
    url='',
    license='LGPL',
    author='Miphreal Adler',
    author_email='miphreal@gmail.com',
    description=''
)
